@@ -61,9 +61,8 @@ class BrailleGenerator(braille_generator.BrailleGenerator):
         imageLink = None
         role = args.get('role', obj.getRole())
         if role == pyatspi.ROLE_IMAGE:
-            imageLink = self._script.getAncestor(obj,
-                                                 [pyatspi.ROLE_LINK],
-                                                 [pyatspi.ROLE_DOCUMENT_FRAME])
+            imageLink = self._script.utilities.ancestorWithRole(
+                obj, [pyatspi.ROLE_LINK], [pyatspi.ROLE_DOCUMENT_FRAME])
         return imageLink
 
     def _generateRoleName(self, obj, **args):
@@ -116,7 +115,7 @@ class BrailleGenerator(braille_generator.BrailleGenerator):
             result.extend(braille_generator.BrailleGenerator._generateName(
                               self, obj, **args))
         if not result and role == pyatspi.ROLE_LIST_ITEM:
-            result.append(self._script.expandEOCs(obj))
+            result.append(self._script.utilities.expandEOCs(obj))
 
         link = None
         if role == pyatspi.ROLE_LINK:
@@ -189,8 +188,13 @@ class BrailleGenerator(braille_generator.BrailleGenerator):
         # relationship.  But, they will make their names be
         # the string of the thing labelling them.
         #
+        # And certain entries (e.g. in the Downloads window) lack a proper
+        # label, instead prefering to display text within the entry which
+        # is deleted when the widget gets focus. We want to treat this
+        # pseudo label as the actual label.
+        #
         if not len(result) \
-           and role == pyatspi.ROLE_COMBO_BOX \
+           and role in [pyatspi.ROLE_COMBO_BOX, pyatspi.ROLE_ENTRY] \
            and not self._script.inDocumentContent(obj):
             result.append(obj.name)
 
@@ -200,7 +204,7 @@ class BrailleGenerator(braille_generator.BrailleGenerator):
         if not len(result):
             parent = obj.parent
             if parent and parent.getRole() == pyatspi.ROLE_AUTOCOMPLETE:
-                label = self._script.getDisplayedLabel(parent)
+                label = self._script.utilities.displayedLabel(parent)
                 if not label or not len(label):
                     label = parent.name
                 result.append(label)
@@ -210,7 +214,7 @@ class BrailleGenerator(braille_generator.BrailleGenerator):
     def _generateExpandedEOCs(self, obj, **args):
         """Returns the expanded embedded object characters for an object."""
         result = []
-        text = self._script.expandEOCs(obj)
+        text = self._script.utilities.expandEOCs(obj)
         if text:
             result.append(text)
         return result
@@ -241,14 +245,18 @@ class BrailleGenerator(braille_generator.BrailleGenerator):
             self._script.isAriaWidget(obj) \
             or ((obj.getRole() == pyatspi.ROLE_LIST) \
                 and (not obj.getState().contains(pyatspi.STATE_FOCUSABLE)))
+
+        oldRole = None
+        if self._script.utilities.isEntry(obj):
+            oldRole = self._overrideRole(pyatspi.ROLE_ENTRY, args)
+
         # Treat menu items in collapsed combo boxes as if the combo box
         # had focus. This will make things more consistent with how we
         # present combo boxes outside of Gecko.
         #
         if obj.getRole() == pyatspi.ROLE_MENU_ITEM:
-            comboBox = self._script.getAncestor(obj,
-                                                [pyatspi.ROLE_COMBO_BOX],
-                                                [pyatspi.ROLE_FRAME])
+            comboBox = self._script.utilities.ancestorWithRole(
+                obj, [pyatspi.ROLE_COMBO_BOX], [pyatspi.ROLE_FRAME])
             if comboBox \
                and not comboBox.getState().contains(pyatspi.STATE_EXPANDED):
                 obj = comboBox
@@ -256,4 +264,6 @@ class BrailleGenerator(braille_generator.BrailleGenerator):
                           generateBraille(self, obj, **args))
         del args['includeContext']
         del args['useDefaultFormatting']
+        if oldRole:
+            self._restoreRole(oldRole, args)
         return result
