@@ -196,6 +196,8 @@ userCustomizableSettings = [
     "flashIsPersistent",
     "flashVerbosityLevel",
     "messageVerbosityLevel",
+    "presentDateFormat",
+    "presentTimeFormat",
 ]
 
 # The name of the module that hold the user interface for the main window
@@ -361,11 +363,11 @@ silenceSpeech           = False
 # Settings that apply to the particular speech engine to
 # use as well details on the default voices to use.
 #
-speechFactoryModules    = ["espeechfactory", \
-                           "gnomespeechfactory", \
-                           "openttsfactory", \
-                           "speechdispatcherfactory"]
-speechServerFactory     = "gnomespeechfactory"
+speechFactoryModules    = ["gnomespeechfactory",
+                           "openttsfactory",
+                           "speechdispatcherfactory",
+                           "espeechfactory"]
+speechServerFactory     = "speechdispatcherfactory"
 speechServerInfo        = None # None means let the factory decide.
 
 DEFAULT_VOICE           = "default"
@@ -883,7 +885,7 @@ useBlockPreventor       = False
 # If True, we use the bonobo main loop provided by bonobo to handle
 # all events in atspi.py.  If False, we create our own loop.
 #
-useBonoboMain           = True
+useBonoboMain           = False
 
 # If True, we handle events asynchronously - our normal mode of
 # queueing events and processing them later on the gidle thread.
@@ -1155,6 +1157,10 @@ setScriptMapping(re.compile('Shredder'), "Thunderbird")
 #
 setScriptMapping(re.compile(_('Mail/News')), "Thunderbird")
 
+# This is for a 3.1 version of Thunderbird, which chooses to now call itself
+# by a different name, Lanikai.
+setScriptMapping(re.compile('Lanikai'), "Thunderbird")
+
 # Translators: see the regular expression note above.  This is for
 # gnome_segv2, which calls itself bug-buddy in at-poke.
 #
@@ -1484,3 +1490,57 @@ useExperimentalSpeechProsody = True
 # some introduce unnaturally long pauses between requests to speak.
 #
 enablePauseBreaks = True
+
+# Format directives to use in presentTime function.
+# By default we use the time format according to the current locale.
+# These format strings are passed to python's time.strftime function. To see
+# possible directives to embed in the format strings check:
+# http://docs.python.org/library/time.html#time.strftime
+#
+TIME_FORMAT_LOCALE = "%X"
+TIME_FORMAT_24_HMS = "%H:%M:%S"
+# Translators: Orca has a feature to speak the time
+# when the user presses a shortcut key.
+# This is one of the alternative formats that the 
+# user may wish to be presented with.
+#
+TIME_FORMAT_24_HMS_WITH_WORDS = _("%H hours, %M minutes and %S seconds.")
+TIME_FORMAT_24_HM = "%H:%M"
+# Translators: Orca has a feature to speak the time
+# when the user presses a shortcut key.
+# This is one of the alternative formats that the 
+# user may wish to be presented with.
+#
+TIME_FORMAT_24_HM_WITH_WORDS = _("%H hours and %M minutes.")
+TIME_FORMAT_CUSTOM = "%X"
+presentTimeFormat = TIME_FORMAT_LOCALE
+
+# Format directives to use in presentDate function.
+# By default we use the date format according to the current locale.
+# These format strings are passed to python's time.strftime function. To see
+# possible directives to embed in the format strings check:
+# http://docs.python.org/library/time.html#time.strftime
+#
+DATE_FORMAT_LOCALE = "%x"
+DATE_FORMAT_NUMBERS_DM = "%d/%m"
+DATE_FORMAT_NUMBERS_MD = "%m/%d"
+DATE_FORMAT_NUMBERS_DMY = "%d/%m/%Y"
+DATE_FORMAT_NUMBERS_MDY = "%m/%d/%Y"
+DATE_FORMAT_NUMBERS_YMD = "%Y/%m/%d"
+DATE_FORMAT_FULL_DM = "%A, %d %B"
+DATE_FORMAT_FULL_MD = "%A, %B %d"
+DATE_FORMAT_FULL_DMY = "%A, %d %B, %Y"
+DATE_FORMAT_FULL_MDY = "%A, %B %d, %Y"
+DATE_FORMAT_FULL_YMD = "%Y. %B %d, %A."
+DATE_FORMAT_ABBREVIATED_DM = "%a, %d %b"
+DATE_FORMAT_ABBREVIATED_MD = "%a, %b %d"
+DATE_FORMAT_ABBREVIATED_DMY = "%a, %d %b, %Y"
+DATE_FORMAT_ABBREVIATED_MDY = "%a, %b %d, %Y"
+DATE_FORMAT_ABBREVIATED_YMD = "%Y. %b %d, %a."
+
+# To keep Orca from spitting up upon launch.
+#
+DATE_FORMAT_WITH_LONG_NAMES = DATE_FORMAT_FULL_DMY
+DATE_FORMAT_WITH_SHORT_NAMES = DATE_FORMAT_ABBREVIATED_DMY
+
+presentDateFormat = DATE_FORMAT_LOCALE
